package PureMessage::Policy::Bundle::CantScanConfig;

use strict;
use PureMessage::Config qw(expand_filename);
use base qw(PureMessage::Config::Validator Exporter);

our @EXPORT_OK = qw(%config %scan_failed);
our %config;
our %scan_failed;

my %nodes = (
    # OPTION_NAME                       => [ TYPE, DEFAULT ],
    'scan_failed_action#Encrypted.id'           => ['#str'],
    'scan_failed_action#Encrypted.description'  => ['#str'],
    'scan_failed_action#Encrypted.action'       => ['#enum allow deny', 'allow'],
    'scan_failed_action#Encrypted.template'     => ['#str'],

    'scan_failed_action#Corrupt.id'           => ['#str'],
    'scan_failed_action#Corrupt.description'  => ['#str'],
    'scan_failed_action#Corrupt.action'       => ['#enum allow deny', 'allow'],
    'scan_failed_action#Corrupt.template'     => ['#str'],

    'scan_failed_action#Timeout.id'           => ['#str'],
    'scan_failed_action#Timeout.description'  => ['#str'],
    'scan_failed_action#Timeout.action'       => ['#enum allow deny', 'allow'],
    'scan_failed_action#Timeout.template'     => ['#str'],

    'scan_failed_action#NotSupported.id'           => ['#str'],
    'scan_failed_action#NotSupported.description'  => ['#str'],
    'scan_failed_action#NotSupported.action'       => ['#enum allow deny', 'allow'],
    'scan_failed_action#NotSupported.template'     => ['#str'],

    'scan_failed_action#ScanAborted.id'           => ['#str'],
    'scan_failed_action#ScanAborted.description'  => ['#str'],
    'scan_failed_action#ScanAborted.action'       => ['#enum allow deny', 'allow'],
    'scan_failed_action#ScanAborted.template'     => ['#str'],

    'scan_failed_action#ByteLimitExceeded.id'           => ['#str'],
    'scan_failed_action#ByteLimitExceeded.description'  => ['#str'],
    'scan_failed_action#ByteLimitExceeded.action'       => ['#enum allow deny', 'allow'],
    'scan_failed_action#ByteLimitExceeded.template'     => ['#str'],

    'scan_failed_action#RecursionLimitExceeded.id'           => ['#str'],
    'scan_failed_action#RecursionLimitExceeded.description'  => ['#str'],
    'scan_failed_action#RecursionLimitExceeded.action'       => ['#enum allow deny', 'allow'],
    'scan_failed_action#RecursionLimitExceeded.template'     => ['#str'],

    'scan_failed_action#CreditCardNumberLimit.id'           => ['#str'],
    'scan_failed_action#CreditCardNumberLimit.description'  => ['#str'],
    'scan_failed_action#CreditCardNumberLimit.action'       => ['#enum allow deny', 'allow'],
    'scan_failed_action#CreditCardNumberLimit.template'     => ['#str'],
);

refresh();

sub refresh {
    require PureMessage::Config::Validator;

    my $p = ActiveState::Config->new(strict => 1, allow_includes => 1);
    PureMessage::Policy::Bundle::CantScanConfig->validate($p);

    $p->parse_file(expand_filename("etc/scanlimit.d/phrase.conf"), @_);

    $config{phrase} = $p->as_hash;

    my $q = ActiveState::Config->new(strict => 1, allow_includes => 1);
    PureMessage::Policy::Bundle::CantScanConfig->validate($q);

    $q->parse_file(expand_filename("etc/scanlimit.d/tft.conf"), @_);

    $config{tft} = $q->as_hash;

    foreach my $type (keys %config) {
        $scan_failed{$type} = {};
        foreach my $scan_type (keys %{$config{$type}->{scan_failed_action}}) {
            my $id = $config{$type}->{scan_failed_action}->{$scan_type}->{id};
            $scan_failed{$type}->{$id} = {};
            $scan_failed{$type}->{$id}->{action}   = $config{$type}->{scan_failed_action}->{$scan_type}->{action};
            $scan_failed{$type}->{$id}->{desc}     = $config{$type}->{scan_failed_action}->{$scan_type}->{description};
            $scan_failed{$type}->{$id}->{template} = $config{$type}->{scan_failed_action}->{$scan_type}->{template};
        }
    }
}

sub validate {
    my ($pkg, $p) = @_;

    for my $key (keys %nodes) {
        my ($type, $default, $sub) = @{$nodes{$key}};
        $p->typedecl($key, $type)   if defined $type;
        $p->default($key, $default) if defined $default;
    }

    $pkg->include_sub_validators($p);
}

1;
