package PureMessage::Policy::Bundle::Utils;

use strict;
use warnings;

use PureMessage::Policy::Bundle::CantScanConfig qw(%config %scan_failed);

use PureMessage::Policy::Plugin::Utils qw(expand_localized_tmpl 
                                          replace_nested_part);
use base 'Exporter';

sub cantscan {
    my ($msg, $args) = @_;

    $msg->log("DEBUG", "cantscan");

    my $map = $msg->get_attr('cantscan_map'); 

    if(ref $map ne 'ARRAY') {
        $msg->log("DEBUG", "cantscan map is not available, probably called out of scope");
        return 0;
    }

    # The map is stored in an anonyous array ref, this is because attributes
    # that want to make use of the 'context' architecture in the message object
    # must be stored as such
    my $cantscan_map = $map->[0];
    if (!exists $cantscan_map->{type}) {
        # This should only happen when you call pmx_cantscan not nested
        # inside other tests
        $msg->log("DEBUG", "no type defined in cantscan map");
        return 0;
    }

    my $return_code = 0;


    # Credit Card shares the configuration file with pmx_phrase
    my $type = $cantscan_map->{type} eq 'cc' ? 'phrase' : $cantscan_map->{type};

    my $ids  = $cantscan_map->{ids};

    my %part_id_flag = ();

    foreach my $pair (@{$ids}) {
       
        # If any id pairs were found, something must have failed,
        # so regardless of the conf, cantscan should return true. 
        $return_code = 1;

        my $sav_id  = $pair->{sav_id};
        my $part_id = $pair->{part_id};

        my $scan_failed_action = $scan_failed{$type}->{$sav_id};
        $scan_failed_action->{action} ||= '';
        
        $msg->log("DEBUG", "cantscan type:'$type' PART_ID:'$part_id' SAV_ID:'$sav_id'");
       
        next if ($scan_failed_action->{action} eq 'allow');

        # Set the description in the msg object, so that it is available to expand_templates
        $cantscan_map->{desc} = $scan_failed_action->{desc};
        $msg->set_attr('cantscan_map', [$cantscan_map], 1);

        if ($scan_failed_action->{action} eq 'warn') {
            next if($part_id_flag{$part_id});

            replace_nested_part($msg,
                                $part_id,
                                expand_localized_tmpl($msg, $scan_failed_action->{template}), 
                                \&expand_templates,
                                warn_only=>1
            );

            $part_id_flag{$part_id} = 'warn';
        }
        elsif ($scan_failed_action->{action} eq 'replace') {
            next if($part_id_flag{$part_id} && $part_id_flag{$part_id} eq 'deny');

            replace_nested_part($msg,
                                $part_id,
                                expand_localized_tmpl($msg, $scan_failed_action->{template}), 
                                \&expand_templates,
            );
            $part_id_flag{$part_id} = 'deny';
        }
        else {
            next if($part_id_flag{$part_id} && $part_id_flag{$part_id} eq 'deny');

            # 'deny' is the default behaviour 
             replace_nested_part($msg,
                                $part_id,
                                expand_localized_tmpl($msg, $scan_failed_action->{template}), 
                                \&expand_templates,
            );
            $part_id_flag{$part_id} = 'deny';
        }
    }

    # There were no limits exceeded or the action was 'allow'
    return $return_code;
}

sub expand_templates {
    my ($msg, $tmpl, $part_id) = @_;

    my $map  = $msg->get_attr('cantscan_map'); 
    my $cantscan_map = $map->[0];
    my $desc = $cantscan_map->{desc} || 'No description found for this ID.';

    use PureMessage::Template qw(expand_template);
    $tmpl = expand_template(
        $tmpl,
        vars => {
                header_subject  => scalar $msg->header('subject'),
                qid             => scalar $msg->minfo_value('Queue-Id'),
                desc            => $desc,
        },
    );
    return $tmpl;
}

1;
